<?php

namespace Larayellow\Tag\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Larayellow\Tag\Forms\Tag as TagForm;
use Larayellow\Tag\Http\Requests\TagRequest;
use Larayellow\Tag\Interfaces\TagRepositoryInterface;
use Larayellow\Tag\Repositories\Eloquent\Filters\TagResourceFilter;
use Larayellow\Tag\Repositories\Eloquent\Presenters\TagListPresenter;

/**
 * Resource controller class for tag.
 */
class TagResourceController extends BaseController
{

    /**
     * Initialize tag resource controller.
     *
     *
     * @return null
     */
    public function __construct(TagRepositoryInterface $tag)
    {
        parent::__construct();
        $this->form = TagForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('larayellow.tag.modules'), 'tag', guard_url('tag'));
        $this->repository = $tag;
    }

    /**
     * Display a list of tag.
     *
     * @return Response
     */
    public function index(TagRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(TagResourceFilter::class)
            ->setPresenter(TagListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('tag::tag.names'))
            ->view('tag::tag.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display tag.
     *
     * @param Request $request
     * @param Model   $tag
     *
     * @return Response
     */
    public function show(TagRequest $request, TagRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('tag::tag.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('tag::tag.show')
            ->output();
    }

    /**
     * Show the form for creating a new tag.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(TagRequest $request, TagRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('tag::tag.name'))
            ->view('tag::tag.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new tag.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TagRequest $request, TagRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('tag::tag.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('tag/tag/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/tag/tag'))
                ->redirect();
        }

    }

    /**
     * Show tag for editing.
     *
     * @param Request $request
     * @param Model   $tag
     *
     * @return Response
     */
    public function edit(TagRequest $request, TagRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('tag::tag.name'))
            ->view('tag::tag.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the tag.
     *
     * @param Request $request
     * @param Model   $tag
     *
     * @return Response
     */
    public function update(TagRequest $request, TagRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('tag::tag.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('tag/tag/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('tag/tag/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the tag.
     *
     * @param Model   $tag
     *
     * @return Response
     */
    public function destroy(TagRequest $request, TagRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('tag::tag.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('tag/tag/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('tag/tag/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
